@@ -268,6 +268,19 @@ got_contact_list_attributes_cb (TpConnection *self,
       g_hash_table_insert (self->priv->roster, key, contact);
     }
 
+  /* emit initial set if roster is not empty */
+  if (g_hash_table_size (self->priv->roster) != 0)
+    {
+      GPtrArray *added;
+      GPtrArray *removed;
+
+      added = tp_connection_dup_contact_list (self);
+      removed = g_ptr_array_new ();
+      g_signal_emit_by_name (self, "contact-list-changed", added, removed);
+      g_ptr_array_unref (added);
+      g_ptr_array_unref (removed);
+    }
+
   self->priv->contact_list_state = TP_CONTACT_LIST_STATE_SUCCESS;
   g_object_notify ((GObject *) self, "contact-list-state");
 
@@ -764,8 +777,8 @@ generic_callback (TpConnection *self,
     result = g_simple_async_result_new ((GObject *) self, callback, user_data, \
         tp_connection_##method##_async); \
     \
-    tp_cli_connection_interface_contact_list_call_##method (self, -1, \
-    handles, ##__VA_ARGS__, generic_callback, result, g_object_unref, NULL); \
+    tp_cli_connection_interface_contact_list_call_##method (self, -1, handles, \
+        ##__VA_ARGS__, generic_callback, result, g_object_unref, NULL); \
     g_array_unref (handles); \
   } G_STMT_END
 
@@ -1106,7 +1119,7 @@ tp_connection_get_contact_groups (TpConnection *self)
         tp_connection_##method##_async); \
     \
     tp_cli_connection_interface_contact_groups_call_##method (self, -1, \
-    group, handles, generic_callback, result, g_object_unref, NULL); \
+        group, handles, generic_callback, result, g_object_unref, NULL); \
     g_array_unref (handles); \
   } G_STMT_END
 
